@@ -116,10 +116,18 @@ sort_impl_by_name (gconstpointer a,
 void
 load_installed_portals (gboolean opt_verbose)
 {
-  const char *portal_dir = DATADIR "/xdg-desktop-portal/portals";
-  g_autoptr(GFile) dir = g_file_new_for_path (portal_dir);
+  const char *portal_dir;
+  g_autoptr(GFile) dir = NULL;
   g_autoptr(GFileEnumerator) enumerator = NULL;
 
+  /* We need to override this in the tests */
+  portal_dir = g_getenv ("XDG_DESKTOP_PORTAL_DIR");
+  if (portal_dir == NULL)
+    portal_dir = DATADIR "/xdg-desktop-portal/portals";
+
+  g_debug ("load portals from %s", portal_dir);
+
+  dir = g_file_new_for_path (portal_dir);
   enumerator = g_file_enumerate_children (dir, "*", G_FILE_QUERY_INFO_NONE, NULL, NULL);
 
   if (enumerator == NULL)
@@ -211,3 +219,25 @@ find_portal_implementation (const char *interface)
 
   return NULL;
 }
+
+GPtrArray *
+find_all_portal_implementations (const char *interface)
+{
+  GPtrArray *impls;
+  GList *l;
+
+  impls = g_ptr_array_new ();
+
+  for (l = implementations; l != NULL; l = l->next)
+    {
+      PortalImplementation *impl = l->data;
+
+      if (g_strv_contains ((const char **)impl->interfaces, interface))
+        {
+          g_debug ("Using %s for %s", impl->source, interface);
+          g_ptr_array_add (impls, impl);
+        }
+    }
+
+  return impls;
+}
