@@ -28,8 +28,6 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <stdio.h>
-#include <mntent.h>
-#include <unistd.h>
 #include <sys/vfs.h>
 
 #include <gio/gdesktopappinfo.h>
@@ -131,7 +129,7 @@ struct _XdpAppInfo {
         } flatpak;
       struct
         {
-          int dummy;
+          GKeyFile *keyfile;
         } snap;
     } u;
 };
@@ -165,6 +163,7 @@ xdp_app_info_free (XdpAppInfo *app_info)
       break;
 
     case XDP_APP_INFO_KIND_SNAP:
+      g_clear_pointer (&app_info->u.snap.keyfile, g_key_file_free);
       break;
 
     case XDP_APP_INFO_KIND_HOST:
@@ -208,10 +207,27 @@ xdp_app_info_load_app_info (XdpAppInfo *app_info)
 
   g_return_val_if_fail (app_info != NULL, NULL);
 
-  if (app_info->id[0] == '\0')
-    return NULL;
+  switch (app_info->kind)
+    {
+    case XDP_APP_INFO_KIND_FLATPAK:
+      desktop_id = g_strconcat (app_info->id, ".desktop", NULL);
+      break;
 
-  desktop_id = g_strconcat (app_info->id, ".desktop", NULL);
+    case XDP_APP_INFO_KIND_SNAP:
+      desktop_id = g_key_file_get_string (app_info->u.snap.keyfile,
+                                          SNAP_METADATA_GROUP_INFO,
+                                          SNAP_METADATA_KEY_DESKTOP_FILE,
+                                          NULL);
+      break;
+
+    case XDP_APP_INFO_KIND_HOST:
+    default:
+      desktop_id = NULL;
+      break;
+    }
+
+  if (desktop_id == NULL)
+    return NULL;
 
   return G_APP_INFO (g_desktop_app_info_new (desktop_id));
 }
@@ -340,7 +356,9 @@ xdp_app_info_has_network (XdpAppInfo *app_info)
       break;
 
     case XDP_APP_INFO_KIND_SNAP:
-      has_network = TRUE; /* FIXME */
+      has_network = g_key_file_get_boolean (app_info->u.snap.keyfile,
+                                            SNAP_METADATA_GROUP_INFO,
+                                            SNAP_METADATA_KEY_NETWORK, NULL);
       break;
 
     case XDP_APP_INFO_KIND_HOST:
@@ -469,133 +487,132 @@ parse_app_info_from_flatpak_info (int pid, GError **error)
   return g_steal_pointer (&app_info);
 }
 
-static gboolean
-aa_is_enabled (void)
+int
+_xdp_parse_cgroup_file (FILE *f, gboolean *is_snap)
 {
-  static int apparmor_enabled = -1;
-  struct stat statbuf;
-  struct mntent *mntpt;
-  FILE *mntfile;
+  ssize_t n;
+  g_autofree char *id = NULL;
+  g_autofree char *controller = NULL;
+  g_autofree char *cgroup = NULL;
+  size_t id_len = 0, controller_len = 0, cgroup_len = 0;
 
-  if (apparmor_enabled >= 0)
-    return apparmor_enabled;
+  g_return_val_if_fail(f != NULL, -1);
+  g_return_val_if_fail(is_snap != NULL, -1);
 
-  apparmor_enabled = FALSE;
-
-  mntfile = setmntent ("/proc/mounts", "r");
-  if (!mntfile)
-    return FALSE;
-
-  while ((mntpt = getmntent (mntfile)))
+  *is_snap = FALSE;
+  do
     {
-      g_autofree char *proposed = NULL;
+      n = getdelim (&id, &id_len, ':', f);
+      if (n == -1) break;
+      n = getdelim (&controller, &controller_len, ':', f);
+      if (n == -1) break;
+      n = getdelim (&cgroup, &cgroup_len, '\n', f);
+      if (n == -1) break;
 
-      if (strcmp (mntpt->mnt_type, "securityfs") != 0)
-        continue;
-
-      proposed = g_strdup_printf ("%s/apparmor", mntpt->mnt_dir);
-      if (stat (proposed, &statbuf) == 0)
+      /* Only consider the freezer, systemd group or unified cgroup
+       * hierarchies */
+      if ((!strcmp (controller, "freezer:") != 0 ||
+           !strcmp (controller, "name=systemd:") != 0 ||
+           !strcmp (controller, ":") != 0) &&
+          strstr (cgroup, "/snap.") != NULL)
         {
-          apparmor_enabled = TRUE;
+          *is_snap = TRUE;
           break;
         }
     }
+  while (n >= 0);
 
-  endmntent (mntfile);
+  if (n < 0 && !feof(f)) return -1;
 
-  return apparmor_enabled;
+  return 0;
 }
 
-#define UNCONFINED		"unconfined"
-#define UNCONFINED_SIZE		strlen(UNCONFINED)
-
 static gboolean
-parse_unconfined (char *con, int size)
+pid_is_snap (pid_t pid, GError **error)
 {
-  return size == UNCONFINED_SIZE && strncmp (con, UNCONFINED, UNCONFINED_SIZE) == 0;
-}
+  g_autofree char *cgroup_path = NULL;;
+  int fd;
+  FILE *f = NULL;
+  gboolean is_snap = FALSE;
+  int err = 0;
 
-static char *
-aa_splitcon (char *con, char **mode)
-{
-  char *label = NULL;
-  char *mode_str = NULL;
-  char *newline = NULL;
-  int size = strlen (con);
+  g_return_val_if_fail(pid > 0, FALSE);
 
-  if (size == 0)
-    return NULL;
-
-  /* Strip newline */
-  if (con[size - 1] == '\n')
+  cgroup_path = g_strdup_printf ("/proc/%u/cgroup", (guint) pid);
+  fd = open (cgroup_path, O_RDONLY | O_CLOEXEC | O_NOCTTY);
+  if (fd == -1)
     {
-      newline = &con[size - 1];
-      size--;
+      err = errno;
+      goto end;
     }
 
-  if (parse_unconfined (con, size))
+  f = fdopen (fd, "r");
+  if (f == NULL)
     {
-      label = con;
+      err = errno;
+      goto end;
     }
-  else if (size > 3 && con[size - 1] == ')')
-    {
-      int pos = size - 2;
 
-      while (pos > 0 && !(con[pos] == ' ' && con[pos + 1] == '('))
-        pos--;
+  fd = -1; /* fd is now owned by f */
 
-      if (pos > 0)
-        {
-          con[pos] = 0; /* overwrite ' ' */
-          con[size - 1] = 0; /* overwrite trailing ) */
-          mode_str = &con[pos + 2]; /* skip '(' */
-          label = con;
-        }
-    }
+  if (_xdp_parse_cgroup_file (f, &is_snap) == -1)
+    err = errno;
 
-  if (label && newline)
-    *newline = 0; /* overwrite '\n', if requested, on success */
-  if (mode)
-    *mode = mode_str;
+  fclose (f);
 
-  return label;
+end:
+  /* Silence ENOENT, treating it as "not a snap" */
+  if (err != 0 && err != ENOENT)
+    {
+      g_set_error (error, G_IO_ERROR, g_io_error_from_errno (err),
+                   "Could not parse cgroup info for pid %u: %s", (guint) pid,
+                   g_strerror (err));
+    }
+  return is_snap;
 }
 
+/* Returns NULL with error set on failure, NULL with no error set if not a snap, and app-info otherwise */
 static XdpAppInfo *
-parse_app_info_from_security_label (const char *security_label)
+parse_app_info_from_snap (pid_t pid, GError **error)
 {
-  char *label, *dot;
-  g_autofree char *snap_name = NULL;
+  g_autoptr(GError) local_error = NULL;
+  g_autofree char *pid_str = NULL;
+  const char *argv[] = { "snap", "routine", "portal-info", NULL, NULL };
+  g_autofree char *output = NULL;
+  g_autoptr(GKeyFile) metadata = NULL;
   g_autoptr(XdpAppInfo) app_info = NULL;
+  g_autofree char *snap_name = NULL;
 
-  /* Snap confinement requires AppArmor */
-  if (aa_is_enabled ())
+  /* Check the process's cgroup membership to fail quickly for non-snaps */
+  if (!pid_is_snap (pid, error)) return NULL;
+
+  pid_str = g_strdup_printf ("%u", (guint) pid);
+  argv[3] = pid_str;
+  if (!xdp_spawnv (NULL, &output, 0, error, argv))
     {
-      /* Parse the security label as an AppArmor context.  We take a copy
-       * of the string because aa_splitcon modifies its argument. */
-      g_autofree char *security_label_copy = g_strdup (security_label);
-
-      label = aa_splitcon (security_label_copy, NULL);
-      if (label && g_str_has_prefix (label, "snap."))
-        {
-          /* If the label belongs to a snap, it will be of the form
-           * snap.$PACKAGE.$APPLICATION.  We want to extract the package
-           * name */
-
-          label += 5;
-          dot = strchr (label, '.');
-          if (!dot)
-            return NULL;
-          snap_name = g_strndup (label, dot - label);
-
-          app_info = xdp_app_info_new (XDP_APP_INFO_KIND_SNAP);
-          app_info->id = g_strconcat ("snap.", snap_name, NULL);
-
-          return g_steal_pointer (&app_info);
-        }
+      return NULL;
     }
 
-  return NULL;
+  metadata = g_key_file_new ();
+  if (!g_key_file_load_from_data (metadata, output, -1, G_KEY_FILE_NONE, &local_error))
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "Can't read snap info for pid %u: %s", pid, local_error->message);
+      return NULL;
+    }
+
+  snap_name = g_key_file_get_string (metadata, SNAP_METADATA_GROUP_INFO,
+                                     SNAP_METADATA_KEY_INSTANCE_NAME, error);
+  if (snap_name == NULL)
+    {
+      return NULL;
+    }
+
+  app_info = xdp_app_info_new (XDP_APP_INFO_KIND_SNAP);
+  app_info->id = g_strconcat ("snap.", snap_name, NULL);
+  app_info->u.snap.keyfile = g_steal_pointer (&metadata);
+
+  return g_steal_pointer (&app_info);
 }
 
 
@@ -606,8 +623,6 @@ xdp_get_app_info_from_pid (pid_t pid,
   g_autoptr(XdpAppInfo) app_info = NULL;
   g_autoptr(GError) local_error = NULL;
 
-  /* TODO: Handle snap support via apparmor here */
-
   app_info = parse_app_info_from_flatpak_info (pid, &local_error);
   if (app_info == NULL && local_error)
     {
@@ -615,6 +630,16 @@ xdp_get_app_info_from_pid (pid_t pid,
       return NULL;
     }
 
+  if (app_info == NULL)
+    {
+      app_info = parse_app_info_from_snap (pid, &local_error);
+      if (app_info == NULL && local_error)
+        {
+          g_propagate_error (error, g_steal_pointer (&local_error));
+          return NULL;
+        }
+    }
+
   if (app_info == NULL)
     app_info = xdp_app_info_new_host ();
 
@@ -648,10 +673,6 @@ xdp_connection_lookup_app_info_sync (GDBusConnection       *connection,
   g_autoptr(GDBusMessage) reply = NULL;
   g_autoptr(XdpAppInfo) app_info = NULL;
   GVariant *body;
-  g_autoptr(GVariantIter) iter = NULL;
-  const char *key;
-  GVariant *value;
-  g_autofree char *security_label = NULL;
   guint32 pid = 0;
 
   app_info = lookup_cached_app_info_by_sender (sender);
@@ -661,7 +682,7 @@ xdp_connection_lookup_app_info_sync (GDBusConnection       *connection,
   msg = g_dbus_message_new_method_call (DBUS_NAME_DBUS,
                                         DBUS_PATH_DBUS,
                                         DBUS_INTERFACE_DBUS,
-                                        "GetConnectionCredentials");
+                                        "GetConnectionUnixProcessID");
   g_dbus_message_set_body (msg, g_variant_new ("(s)", sender));
 
   reply = g_dbus_connection_send_message_with_reply_sync (connection, msg,
@@ -680,37 +701,11 @@ xdp_connection_lookup_app_info_sync (GDBusConnection       *connection,
     }
 
   body = g_dbus_message_get_body (reply);
+  g_variant_get (body, "(u)", &pid);
 
-  g_variant_get (body, "(a{sv})", &iter);
-  while (g_variant_iter_loop (iter, "{&sv}", &key, &value))
-    {
-      if (strcmp (key, "ProcessID") == 0)
-        pid = g_variant_get_uint32 (value);
-      else if (strcmp (key, "LinuxSecurityLabel") == 0)
-        {
-          g_clear_pointer (&security_label, g_free);
-          security_label = g_variant_dup_bytestring (value, NULL);
-        }
-    }
-
-  if (app_info == NULL && security_label != NULL)
-    {
-      app_info = parse_app_info_from_security_label (security_label);
-    }
-
+  app_info = xdp_get_app_info_from_pid (pid, error);
   if (app_info == NULL)
-    {
-      g_autoptr(GError) local_error = NULL;
-      app_info = parse_app_info_from_flatpak_info (pid, &local_error);
-      if (app_info == NULL && local_error)
-        {
-          g_propagate_error (error, g_steal_pointer (&local_error));
-          return NULL;
-        }
-    }
-
-  if (app_info == NULL)
-    app_info = xdp_app_info_new_host ();
+    return NULL;
 
   G_LOCK (app_infos);
   ensure_app_info_by_unique_name ();
@@ -889,6 +884,35 @@ verify_proc_self_fd (XdpAppInfo *app_info,
   return xdp_app_info_remap_path (app_info, path_buffer);
 }
 
+static char *documents_mountpoint = NULL;
+
+void
+xdp_set_documents_mountpoint (const char *path)
+{
+  g_clear_pointer (&documents_mountpoint, g_free);
+  documents_mountpoint = g_strdup (path);
+}
+
+/* alternate_document_path converts a file path  */
+char *
+xdp_get_alternate_document_path (const char *path, const char *app_id)
+{
+  int len;
+
+  /* If we don't know where the document portal is mounted, then there
+   * is no alternate path */
+  if (documents_mountpoint == NULL)
+    return NULL;
+
+  /* If the path is not within the document portal, then there is no
+   * alternative path */
+  len = strlen (documents_mountpoint);
+  if (!g_str_has_prefix (path, documents_mountpoint) || path[len] != '/')
+    return NULL;
+
+  return g_strconcat (documents_mountpoint, "/by-app/", app_id, &path[len], NULL);
+}
+
 char *
 xdp_app_info_get_path_for_fd (XdpAppInfo *app_info,
                               int fd,
@@ -986,8 +1010,22 @@ xdp_app_info_get_path_for_fd (XdpAppInfo *app_info,
       st_buf->st_dev != real_st_buf.st_dev ||
       st_buf->st_ino != real_st_buf.st_ino)
     {
-      /* Different files on the inside and the outside, reject the request */
-      return NULL;
+      /* If the path is provided by the document portal, the inode
+         number will not match, due to only a subtree being mounted in
+         the sandbox.  So we check to see if the equivalent path
+         within that subtree matches our file descriptor.
+
+         If the alternate path doesn't match either, then we treat it
+         as a failure.
+      */
+      g_autofree char *alt_path = NULL;
+      alt_path = xdp_get_alternate_document_path (path, xdp_app_info_get_id (app_info));
+
+      if (alt_path == NULL ||
+          stat (alt_path, &real_st_buf) < 0 ||
+          st_buf->st_dev != real_st_buf.st_dev ||
+          st_buf->st_ino != real_st_buf.st_ino)
+        return NULL;
     }
 
   if (writable_out)
