@@ -1,5 +1,6 @@
 #include <config.h>
-#include <config.h>
+#include <string.h>
+#include <locale.h>
 
 #include <gio/gio.h>
 
@@ -39,6 +40,17 @@ static GSubprocess *backends;
 XdpImplPermissionStore *permission_store;
 XdpImplLockdown *lockdown;
 
+int
+xdup (int oldfd)
+{
+  int newfd = dup (oldfd);
+
+  if (newfd < 0)
+    g_error ("Unable to duplicate fd %d: %s", oldfd, g_strerror (errno));
+
+  return newfd;
+}
+
 static void
 name_appeared_cb (GDBusConnection *bus,
                   const char *name,
@@ -72,18 +84,56 @@ timeout_cb (gpointer data)
   return G_SOURCE_REMOVE;
 }
 
+static void
+update_data_dirs (void)
+{
+  const char *data_dirs;
+  gssize len = 0;
+  GString *str;
+  char *new_val;
+
+  data_dirs = g_getenv ("XDG_DATA_DIRS");
+  if (data_dirs != NULL &&
+      strstr (data_dirs, "/usr/share") != NULL)
+    {
+      return;
+    }
+
+  if (data_dirs != NULL)
+    {
+      len = strlen (data_dirs);
+      if (data_dirs[len] == ':')
+        len--;
+    }
+
+  str = g_string_new_len (data_dirs, len);
+  if (str->len > 0)
+    g_string_append_c (str, ':');
+  g_string_append (str, "/usr/local/share/:/usr/share/");
+  new_val = g_string_free (str, FALSE);
+
+  g_debug ("Setting XDG_DATA_DIRS to %s", new_val);
+  g_setenv ("XDG_DATA_DIRS", new_val, TRUE);
+  /* new_val is leaked */
+}
+
 static void
 global_setup (void)
 {
   GError *error = NULL;
+  g_autofree gchar *backends_executable = NULL;
   g_autofree gchar *services = NULL;
   g_autofree gchar *portal_dir = NULL;
+  g_autofree gchar *argv0 = NULL;
   g_autoptr(GSubprocessLauncher) launcher = NULL;
   guint name_timeout;
   const char *argv[4];
   GQuark portal_errors G_GNUC_UNUSED;
   static gboolean name_appeared;
   guint watch;
+  guint timeout_mult = 1;
+
+  update_data_dirs ();
 
   g_mkdtemp (outdir);
   g_print ("outdir: %s\n", outdir);
@@ -96,6 +146,9 @@ global_setup (void)
   g_test_dbus_add_service_dir (dbus, services);
   g_test_dbus_up (dbus);
 
+  if (g_getenv ("TEST_IN_CI"))
+    timeout_mult = 10;
+
   /* g_test_dbus_up unsets this, so re-set */
   g_setenv ("XDG_RUNTIME_DIR", outdir, TRUE);
 
@@ -117,8 +170,10 @@ global_setup (void)
   g_subprocess_launcher_setenv (launcher, "DBUS_SESSION_BUS_ADDRESS", g_test_dbus_get_bus_address (dbus), TRUE);
   g_subprocess_launcher_setenv (launcher, "XDG_DATA_HOME", outdir, TRUE);
   g_subprocess_launcher_setenv (launcher, "PATH", g_getenv ("PATH"), TRUE);
- 
-  argv[0] = "test-backends";
+  g_subprocess_launcher_take_stdout_fd (launcher, xdup (STDERR_FILENO));
+
+  backends_executable = g_test_build_filename (G_TEST_BUILT, "test-backends", NULL);
+  argv[0] = backends_executable;
   argv[1] = g_test_verbose () ? "--verbose" : NULL;
   argv[2] = NULL;
 
@@ -127,7 +182,7 @@ global_setup (void)
   backends = g_subprocess_launcher_spawnv (launcher, argv, &error);
   g_assert_no_error (error);
 
-  name_timeout = g_timeout_add (1000, timeout_cb, "Failed to launch test-backends");
+  name_timeout = g_timeout_add (1000 * timeout_mult, timeout_cb, "Failed to launch test-backends");
 
   while (!name_appeared)
     g_main_context_iteration (NULL, TRUE);
@@ -154,17 +209,24 @@ global_setup (void)
   g_subprocess_launcher_setenv (launcher, "XDG_DESKTOP_PORTAL_DIR", portal_dir, TRUE);
   g_subprocess_launcher_setenv (launcher, "XDG_DATA_HOME", outdir, TRUE);
   g_subprocess_launcher_setenv (launcher, "PATH", g_getenv ("PATH"), TRUE);
- 
-  argv[0] = "xdg-desktop-portal";
+  g_subprocess_launcher_take_stdout_fd (launcher, xdup (STDERR_FILENO));
+
+  if (g_getenv ("XDP_UNINSTALLED") != NULL)
+    argv0 = g_test_build_filename (G_TEST_BUILT, "..", "xdg-desktop-portal", NULL);
+  else
+    argv0 = g_strdup (LIBEXECDIR "/xdg-desktop-portal");
+
+  argv[0] = argv0;
   argv[1] = g_test_verbose () ? "--verbose" : NULL;
   argv[2] = NULL;
 
-  g_print ("launching xdg-desktop-portal\n");
+  g_print ("launching %s\n", argv0);
 
   portals = g_subprocess_launcher_spawnv (launcher, argv, &error);
   g_assert_no_error (error);
+  g_clear_pointer (&argv0, g_free);
 
-  name_timeout = g_timeout_add (1000, timeout_cb, "Failed to launch xdg-desktop-portal");
+  name_timeout = g_timeout_add (1000 * timeout_mult, timeout_cb, "Failed to launch xdg-desktop-portal");
 
   while (!name_appeared)
     g_main_context_iteration (NULL, TRUE);
@@ -188,18 +250,24 @@ global_setup (void)
   g_subprocess_launcher_setenv (launcher, "DBUS_SESSION_BUS_ADDRESS", g_test_dbus_get_bus_address (dbus), TRUE);
   g_subprocess_launcher_setenv (launcher, "XDG_DATA_HOME", outdir, TRUE);
   g_subprocess_launcher_setenv (launcher, "PATH", g_getenv ("PATH"), TRUE);
- 
-  argv[0] = "xdg-permission-store";
+  g_subprocess_launcher_take_stdout_fd (launcher, xdup (STDERR_FILENO));
+
+  if (g_getenv ("XDP_UNINSTALLED") != NULL)
+    argv0 = g_test_build_filename (G_TEST_BUILT, "..", "xdg-permission-store", NULL);
+  else
+    argv0 = g_strdup (LIBEXECDIR "/xdg-permission-store");
+
+  argv[0] = argv0;
   argv[1] = "--replace";
   argv[2] = g_test_verbose () ? "--verbose" : NULL;
   argv[3] = NULL;
 
-  g_print ("launching xdg-permission-store");
+  g_print ("launching %s\n", argv0);
 
   portals = g_subprocess_launcher_spawnv (launcher, argv, &error);
   g_assert_no_error (error);
 
-  name_timeout = g_timeout_add (1000, timeout_cb, "Failed to launch xdg-permission-store");
+  name_timeout = g_timeout_add (1000 * timeout_mult, timeout_cb, "Failed to launch xdg-permission-store");
 
   while (!name_appeared)
     g_main_context_iteration (NULL, TRUE);
@@ -306,7 +374,7 @@ DEFINE_TEST_EXISTS(account, ACCOUNT, 1)
 DEFINE_TEST_EXISTS(background, BACKGROUND, 1)
 DEFINE_TEST_EXISTS(camera, CAMERA, 1)
 DEFINE_TEST_EXISTS(email, EMAIL, 3)
-DEFINE_TEST_EXISTS(file_chooser, FILE_CHOOSER, 2)
+DEFINE_TEST_EXISTS(file_chooser, FILE_CHOOSER, 3)
 DEFINE_TEST_EXISTS(game_mode, GAME_MODE, 3)
 DEFINE_TEST_EXISTS(inhibit, INHIBIT, 3)
 DEFINE_TEST_EXISTS(location, LOCATION, 1)
@@ -325,6 +393,8 @@ main (int argc, char **argv)
 {
   int res;
 
+  setlocale (LC_ALL, NULL);
+
   g_test_init (&argc, &argv, NULL);
 
   g_test_add_func ("/portal/account/exists", test_account_exists);
